import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:pmd_lab/components/utils/debounce.dart';
import 'package:pmd_lab/domain/models/card.dart';
import 'package:pmd_lab/presentation/details_page/details_page.dart';
import 'package:pmd_lab/presentation/home_page/bloc/bloc.dart';
import 'package:pmd_lab/presentation/home_page/bloc/events.dart';
import 'package:pmd_lab/presentation/home_page/bloc/state.dart';
import 'package:pmd_lab/repositories/api_interface.dart';
import 'package:pmd_lab/repositories/mock_repository.dart';
import 'package:pmd_lab/repositories/potter_repository.dart';
part 'card.dart';

void main() {
  runApp(const MyApp());
}

class MyApp extends StatelessWidget {
  const MyApp({super.key});

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Flutter Demo',
      theme: ThemeData(
        // colorScheme: ColorScheme.fromSeed(seedColor: Colors.black),                                                                                                                  colorScheme: ColorScheme.fromSeed(seedColor: Colors.deepPurple),
        useMaterial3: true,
      ),
      home: const MyHomePage(title: 'Flutter rocks!'),
    );
  }
}

class MyHomePage extends StatefulWidget {
  const MyHomePage({super.key, required this.title});

  final String title;

  @override
  State<MyHomePage> createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  final Color _color = Colors.orangeAccent;

  @override
  void initState(){
    WidgetsBinding.instance.addPostFrameCallback( (_) {
      ScaffoldMessenger.of(context).showSnackBar(SnackBar(
        content: Text('Hello', style: Theme
            .of(context)
            .textTheme
            .bodyLarge,),
        backgroundColor: Colors.orangeAccent,
        duration: const Duration(seconds: 1),
      ));
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        backgroundColor: _color,
        title: Text(widget.title),
      ),
      body: const Body(),
    );
  }
}

class Body extends StatefulWidget {
  const Body({super.key});

  @override
  State<Body> createState() => _BodyState();
}

class _BodyState extends State<Body> {
  final searchController = TextEditingController();
  final scrollController = ScrollController();
  final ApiInterface repo = PotterRepository();
  late Future<List<CardData>?> data;

  @override
  void initState() {
    WidgetsBinding.instance.addPostFrameCallback((_) {
      context.read<HomeBloc>().add(const HomeLoadDataEvent());
    });
    scrollController.addListener(_onNextPageListener);
    super.initState();
  }
  void _onNextPageListener() {
    if (scrollController.offset > scrollController.position.maxScrollExtent) {
      final bloc = context.read<HomeBloc>();
      if (!bloc.state.isPaginationLoading) {
        bloc.add(HomeLoadDataEvent(
          search: searchController.text,
          nextPage: bloc.state.data?.nextPage,
        ));
      }
    }
  }
  @override
  void dispose(){
    searchController.dispose();
    scrollController.dispose();

    super.dispose();
  }

  @override
  Widget build(BuildContext context) {

    return Padding(
      padding: EdgeInsets.only(top: MediaQuery.of(context).padding.top),
      child: Column(
        children: [
          Padding(
            padding: const EdgeInsets.all(12),
            child: CupertinoSearchTextField(
              controller: searchController,
              onChanged: (search) {
                Debounce.run(() => context.read<HomeBloc>().add(HomeLoadDataEvent(search: search)));
              },
            ),
          ),

          BlocBuilder<HomeBloc, HomeState>(
            builder: (context, state) => state.error != null
              ? Text(
                  state.error ?? '',
                  style: Theme.of(context).textTheme.headlineSmall?.copyWith(color: Colors.red),
                )
              : state.isLoading
                ? const CircularProgressIndicator()
                : Expanded(
                  child: RefreshIndicator(
                    onRefresh: _onRefresh,
                    child: ListView.builder(
                      padding: EdgeInsets.zero,
                      itemCount: state.data?.data?.length ?? 0,
                      itemBuilder: (context, index) {
                        final data = state.data?.data?[index];
                        return data != null
                            ? _Card.fromData(
                          data,
                          onLike: (title, isLiked) => _showSnackBar(context, title, isLiked),
                          onTap: () => _navToDetails(context, data),
                        )
                            : const SizedBox.shrink();
                  },
                ), // ListView.builder
              ), // RefreshIndicator
            ), // Expanded
          ), // BlocBuilder
          BlocBuilder<HomeBloc, HomeState>(
            builder: (context, state) => state.isPaginationLoading
                ? const CircularProgressIndicator()
                : const SizedBox.shrink(),
          )
        ],
      ),
    );
  }

  void _showSnackBar(BuildContext context, String title, bool isLiked) {
    WidgetsBinding.instance.addPostFrameCallback((_) {
      ScaffoldMessenger.of(context).showSnackBar(SnackBar(
        content: Text(
          'Article $title ${isLiked ? 'liked!' : 'disliked :('}',
          style: Theme
              .of(context)
              .textTheme
              .bodyLarge,),
        backgroundColor: Colors.orangeAccent,
        duration: const Duration(seconds: 1),
      ));
    });
  }

  void _navToDetails(BuildContext context, CardData data) {
    Navigator.push(
      context,
      CupertinoPageRoute(builder: (context) => DetailsPage(data)),
    );
  }

  Future<void> _onRefresh() {
    context.read<HomeBloc>().add(HomeLoadDataEvent(search: searchController.text));
    return Future.value(null);
  }

}